"""
"""

import numpy as np
from numpy import radians


AVG_EARTH_RADIUS = 6371000  # in m


def find_closer_then(lat, lon, lat_c, lon_c, llimit, method="haversine"):
    """
    """
    assert lat.shape == lon.shape, "lat & lon must have same shape"
    #assert (len(lat_c) == 1) & (len(lon_c) == 1)

    deg_limit = np.rad2deg(llimit/AVG_EARTH_RADIUS)
    possible = np.nonzero(abs(lat - lat_c) <= deg_limit)
    L = haversine(lat[possible], lon[possible], lat_c, lon_c)
    ind_closer = L<llimit
    L = L[ind_closer]
    ind = []
    # Repeat for each dimension.
    for p in possible:
        ind.append(p[ind_closer])

    return ind, L

def haversine(lat, lon, lat_c, lon_c):

    lat, lon, lat_c, lon_c = map(radians, [lat, lon, lat_c, lon_c])

    dlat = lat - lat_c
    dlon = lon - lon_c
    d = np.sin(dlat / 2) ** 2 + \
            np.cos(lat) * np.cos(lat_c) * np.sin(dlon / 2) ** 2
    h = 2 * AVG_EARTH_RADIUS * np.arcsin(np.sqrt(d))

    return h



__all__ = ['core', 'window_func', 'distance']

from core1D import wmean_1D_serial, wmean_1D, wmean_bandpass_1D_serial, wmean_bandpass_1D
from core2D import wmean_2D_serial, wmean_2D
from core2Dlatlon import wmean_2D_latlon_serial, wmean_2D_latlon



# -*- coding: utf-8 -*-
# cython: profile=True
# filename: maud.pyx

"""

    Wasn't this below supposed to be faster than from i in xrange()?
    #for i from 0 < = i < I:
    #    for j from 0 < = j < J:
"""

import numpy as np
from numpy import ma

import multiprocessing as mp

cimport numpy as np
from libc.math cimport abs, cos, sin, asin, sqrt, M_PI

from maud.cwindow_func_scalar import window_func_scalar
from maud.cdistance import haversine_scalar as _haversine_scalar

try:
    from progressbar import ProgressBar
except:
    print "ProgressBar is not available"


np.import_array()

DTYPE = np.float
ctypedef np.float_t DTYPE_t


from maud.window_func import window_func

def wmean_1D_serial(data, l, t=None, method='hann', axis=0, interp=False):
    """ A moving window mean filter, not necessarily a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        It's not optimized for a regular grid.

        Input:
            - data: np.array or ma.maked_array, nD
            - l: is the size of the filter, in the same unit
                of the t.
            - t: is the scale of the choosed axis, 1D. If not
                defined, it will be considered a sequence.
            - method: ['hann', 'hamming', 'blackman']
                Defines the weight function type
            - axis: Dimension which the filter will be applied
    """
    #assert type(data)) in [np.ndarray, ma.MaskedArray]
    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        print("Will temporary swapaxes")
        data_smooth = wmean_1D_serial(data.swapaxes(0, axis),
            l = l,
            t = t,
            method = method,
            axis = 0,
            interp = interp)

        return data_smooth.swapaxes(0, axis)
    # Below here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
        t = np.arange(data.shape[axis])

    t = t.astype('float64')
    assert t.shape == (data.shape[axis],), "Invalid size of t."

    l = float(l)

    if (data.ndim == 1):
        wfunc = window_func_scalar(method)

        if (type(data) is np.ndarray):
            data_smooth = convolve_1D_array(data, t, l, wfunc)
            return data_smooth

        elif (type(data) is ma.MaskedArray) and (data.ndim == 1):
            d, m = convolve_1D_MA(data.data,
                    ma.getmaskarray(data).astype('int8'),
                    t, l, method, interp)
            return ma.masked_array(d, m)


    if type(data) is np.ndarray:
        data_smooth = np.empty(data.shape)
    else:
        data_smooth = ma.masked_all(data.shape)

    # ----
    if data.ndim > 1:
        for i in range(data.shape[1]):
            data_smooth[:,i] = wmean_1D_serial(data[:,i],
                l = l,
                t = t,
                method = method,
                axis = 0,
                interp = interp)

        return data_smooth
    # ----
    # Here on it is expected data.ndim == 1

    wfunc = window_func(method)

    if interp is True:
        I = range(len(t))
    else:
        (I,) = np.nonzero(~ma.getmaskarray(data))

    for i in I:
        dt = t - t[i]
        w = wfunc(dt, l)
        ind = (w != 0) & (~ma.getmaskarray(data))
        if ind.any():
            tmp = data[ind]*w[ind]
            wsum = w[ind].sum()
            if wsum != 0:
                data_smooth[i] = (tmp).sum()/wsum

    return data_smooth


def wmean_1D(data, l, t=None, method='hann', axis=0, interp=False):
    return wmean_1D_serial(data, l, t, method, axis, interp)


def window_1Dmean(data, double l, t=None, method='hann', axis=0, parallel=True):
    """ A moving window mean filter, not necessarily a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        It's not optimized for a regular grid.

        t is the scale of the choosed axis

        l is the size of the filter.
    """
    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        data_smooth = window_1Dmean(data.swapaxes(0,axis),
            l = l,
            t = t,
            method = method,
            axis=0,
            parallel = parallel)

        return data_smooth.swapaxes(0,axis)
    # Bellow here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
        t = np.arange(data.shape[axis], dtype=np.float)
    else:
        assert t.dtype == np.float, "Var t must be of type np.float"

    # t must has the same shape of data along axis
    assert t.shape == (data.shape[axis],)

    # ----
    weight_func = window_func_scalar(method)

    data_smooth = ma.masked_all(data.shape)

    if data.ndim==1:
        # It's faster than getmaskarray
        (I,) = np.nonzero(np.isfinite(data))

        for i in I:
            #data_smooth[i] = apply_window_1Dmean(data, t[i], t, l, I, weight_func)
            out = apply_window_1Dmean(data, t[i], t, l, I, weight_func)
            if out is not None:
                data_smooth[i] = out

    elif parallel is True:
        npes = 2 * mp.cpu_count()
        pool = mp.Pool(npes)
        results = []
        I = data.shape[1]
        try:
            pbar = ProgressBar(maxval=I).start()
        except:
            pass

        for i in range(I):
            results.append( pool.apply_async( window_1Dmean, \
                           (data[:,i], l, t, method, 0, False)))
        pool.close()
        for i, r in enumerate(results):
            try:
                pbar.update(i)
            except:
                pass
            data_smooth[:,i] = r.get()
        pool.terminate()

    else:
        I = data.shape[1]
        for i in range(I):
            data_smooth[:,i] = window_1Dmean(data[:,i],
                    l = l,
                    t = t,
                    method = method,
                    axis = 0,
                    parallel=parallel)

    return data_smooth


def convolve_1D_array(np.ndarray[DTYPE_t, ndim=1] data,
        np.ndarray[DTYPE_t, ndim=1] t, double l, wfunc):

    cdef unsigned int I = len(t)
    cdef unsigned int i, ii
    cdef double D, W, dt, w
    # The ideal would be use data.shape instead of I.
    cdef np.ndarray[DTYPE_t, ndim=1] data_smooth = np.empty(I)

    #wfunc = window_func_scalar(method)

    for i in xrange(I):
        D = 0
        W = 0
        for ii in xrange(I):
            dt = t[ii] - t[i]
            if abs(dt) <= l:
                w = wfunc(dt, l)
                if w != 0:
                    D += data[ii]*w
                    W += w

        assert (W != 0), "It's an array, must have at least one valid data"
        data_smooth[i] = D/W

    return data_smooth


def convolve_1D_MA(np.ndarray[DTYPE_t, ndim=1] data,
        np.ndarray[np.int8_t, ndim=1] mask,
        np.ndarray[DTYPE_t, ndim=1] t, double l, method, bint interp):

    cdef unsigned int I = len(t)
    cdef unsigned int i, ii
    cdef double D, W, dt, w
    # The ideal would be use data.shape instead of I.
    cdef np.ndarray[DTYPE_t, ndim=1] data_smooth = np.empty(I)
    cdef np.ndarray[np.int8_t, ndim=1] mask_smooth = np.zeros(I, 'int8')

    wfunc = window_func_scalar(method)

    for i in xrange(I):
        if (interp == 0) and (mask[i] == 1):
            mask_smooth[i] = 1
        else:
            D = 0
            W = 0
            for ii in xrange(I):
                if mask[ii] == 0:
                    dt = t[ii] - t[i]
                    if abs(dt) <= l:
                        w = wfunc(dt, l)
                        if w != 0:
                            D += data[ii]*w
                            W += w

            assert (W != 0), "It's an array, must have at least one valid data"
            data_smooth[i] = D/W

    return data_smooth, mask_smooth


def apply_window_1Dmean(np.ndarray[DTYPE_t, ndim=1] data,
        double t0, np.ndarray[DTYPE_t, ndim=1] t, double l,
        np.ndarray[np.int_t, ndim=1] I, weight_func):

    cdef int i
    cdef double D, W
    cdef double dt, w
    #cdef size_t I

    W = 0
    D = 0

    for i in I:
        dt = t[i] - t0
        if abs(dt) <= l:
            w = weight_func(dt, l)
            if w != 0:
                D += data[i] * w
                W += w
    if W != 0:
        return D/W

# ============================================================================
def wmean_2D_serial(x, y, data, l, method='hamming', interp=False):
    """ Temporary solution
    """
    import maud
    return maud.wmean_2D_serial(x, y, data, l, method, interp)

def wmean_2D(x, y, data, l, method='hamming', interp=False):
    """ Temporary solution
    """
    return wmean_2D_serial(x, y, data, l, method, interp)
# ============================================================================
def window_mean_2D_latlon(Lat, Lon, data, l, method='hamming', interp=False):
    """
        Cython version of the window_mean_2D_latlon()

        Input:
          - Lat: 2D array with latitudes
          - Lon: 2D array with longitudes
          - data: An array or a masked array. It must be 2D or 3D. In
	      the case of a 3D, the two last dimensions will be
	      considered as lat and lon, while it will run along the
	      first dimension with the multiprocessing.
          - l: window filter size, in meters
          - method: ['hann', 'hamming', 'blackman']
	      Defines the weight function type

        Output: An array of the same type and dimension of the input
	  data. This will be the low (frequency) pass filtered data,
	  i.e. it will eliminate all short variability

        !!!ATENTION!!!
        - Might be a good idea to eliminate the dependence on
          fluid.
    """

    #if type(data) == dict:
    #    output = {}
    #    for k in data.keys():
    #        output[k] = window_mean_2D_latlon(Lat, Lon, data[k], l, method)
    #    return output

    assert (Lat.ndim == 2) & (Lon.ndim == 2), "Lat and Lon must be 2D array"
    #assert data.ndim == 2, "Sorry, for now I'm only handeling 2D arrays"

    # ==== data is a 2D array ======================================
    if data.ndim == 2:
        if hasattr(data, 'mask'):
            if (data.mask == True).any():
                data_smooth, mask = apply_window_mean_2D_latlon_masked(Lat,
                        Lon, data.data, data.mask.astype('int8'), l, method,
                        interp)
                return ma.masked_array(data_smooth, mask)
            else:
                data_smooth = apply_window_mean_2D_latlon(Lat, Lon, data.data,
                        l, method)
                return ma.array(data_smooth)
        else: # type(data) == np.ndarray:
            return apply_window_mean_2D_latlon(Lat, Lon, data, l, method)

    # ==== data is a 3D array ======================================
    elif data.ndim == 3:
        try:
            from progressbar import ProgressBar
        except:
            print "ProgressBar is not available"

        import multiprocessing as mp
        npes = 2 * mp.cpu_count()
        print " Will work with %s npes" % npes
        data_smooth = ma.empty(data.shape)
        pool = mp.Pool(npes)
        results = []

        N = data.shape[0]

        try:
            pbar = ProgressBar(maxval=N).start()
        except:
            pass

        for n in range(N):
            results.append( pool.apply_async( window_mean_2D_latlon, (Lat, Lon, data[n], l, method, interp) ) )

        for i, r in enumerate(results):
            try:
                pbar.update(i)
            except:
                pass
            data_smooth[i] = r.get()

        return data_smooth

    #if interp == True:
    #    I, J = data.shape
    #    I, J = np.meshgrid(range(I), range(J))
    #    I = I.reshape(-1); J = J.reshape(-1)
    #else:
    #    I, J = np.nonzero(np.isfinite(data))

#        #ind, r = cfind_closer_then(Lat, Lon, Lat[i,j], Lon[i,j], llimit=l)
#        #if len(r) > 0:
#        #    w = weight_func(r, l)
#        #    tmp = data[ind]*w
#        #    wsum = w[np.nonzero(tmp)].sum()
#        #    if wsum != 0:
#        #        data_smooth[i,j] = (tmp).sum()/wsum
#	#elif data.ndim == 3:
#	#    for k in range(data.shape[0]):
#	#        data_smooth[k,i,j] = (data[k][ind]*w).sum()/w[good].sum()
#	#    else:
#	#        data_smooth[k,i,j] = (data[k][ind]*w).sum()/w[good].sum()
#    return data_out
    #return data_smooth

def apply_window_mean_2D_latlon(np.ndarray[DTYPE_t, ndim=2] Lat,
        np.ndarray[DTYPE_t, ndim=2] Lon, np.ndarray[DTYPE_t, ndim=2] data,
        double l, method='hamming'):
    """
    """

    cdef unsigned int i, ii, j, jj
    cdef double r, w
    cdef unsigned int I = data.shape[0]
    cdef unsigned int J = data.shape[1]
    cdef np.ndarray[DTYPE_t, ndim=2] D = np.zeros((I,J))
    cdef np.ndarray[DTYPE_t, ndim=2] W = np.zeros((I,J))

    weight_func = window_func_scalar(method)

    for i in xrange(I):
        for j in xrange(J):
            for ii in xrange(i, I):
                for jj in xrange(j, J):
                    r = _haversine_scalar(Lat[i,j], Lon[i,j],
                            Lat[ii,jj], Lon[ii,jj])
                    if r <= l:
                        w = weight_func(r, l)
                        if w != 0:
                            D[i, j] += data[ii, jj] * w
                            W[i, j] += w

                            if (i != ii) & (j != jj):
                                D[ii, jj] += data[i, j] * w
                                W[ii, jj] += w

    return D/W


def apply_window_mean_2D_latlon_masked(np.ndarray[DTYPE_t, ndim=2] Lat,
		np.ndarray[DTYPE_t, ndim=2] Lon,
		np.ndarray[DTYPE_t, ndim=2] data,
		np.ndarray[np.int8_t, ndim=2] mask,
		double l, method, interp):
#    """
#    """
    weight_func = window_func_scalar(method)

    cdef unsigned int i, ii, j, jj
    cdef double r, w
    cdef unsigned int I = data.shape[0]
    cdef unsigned int J = data.shape[1]
    cdef np.ndarray[DTYPE_t, ndim=2] D = np.zeros((I,J))
    cdef np.ndarray[DTYPE_t, ndim=2] W = np.zeros((I,J))
    cdef np.ndarray[DTYPE_t, ndim=2] data_smooth = np.empty((I,J))
    cdef np.ndarray[np.int8_t, ndim=2] mask_smooth = \
		    np.ones((I,J), dtype=np.int8)

    for i in xrange(I):
        for j in xrange(J):
            if (interp is True) or (mask[i, j] == 0):
                for ii in xrange(i, I):
                    for jj in xrange(j, J):
                        if mask[ii, jj] == 0:
                            r = _haversine_scalar(Lat[i,j], Lon[i,j],
                                    Lat[ii,jj], Lon[ii,jj])
                            if r <= l:
                                w = weight_func(r, l)
                                if w != 0:
                                    if mask[ii, jj] == 0:
                                        D[i, j] += data[ii, jj] * w
                                        W[i, j] += w
                                    if mask[i, j] == 0:
                                        D[ii, jj] += data[i, j] * w
                                        W[ii, jj] += w
    for i in xrange(I):
        for j in xrange(J):
            if W[i, j] != 0:
                data_smooth[i, j] = D[i,j]/W[i,j]
                mask_smooth[i, j] = 0

    return data_smooth, mask_smooth




import numpy as np
from numpy import ma

from window_func import window_func
from distance import haversine

 
def wmean_2D_latlon_serial(lat, lon, data, l, method='hamming',
        interp='False'):
    """
        Right now I'm doing for Masked Arrays only.
        data should be a dictionary with 2D arrays

        Input:
          - Lat: 2D array with latitudes
          - Lon: 2D array with longitudes
          - data: There are two possibilities, it can be an 
            array of at least 2D, or a dictionary where each
            element is an array of at least 2D.
          - l: window filter size, in meters
          - method: weight function type
          - interp: if False, estimate only for the gridponits
              of valid data, i.e. data.mask = output.mask. If
              True, estimate for all gridpoints that has at
              least one valida data point inside the l/2 radius 

        Output:

    """
    #assert ((type(l) == float) or (type(l) == int)), \
    #    "The filter scale (l) must be a float or an int"

    #if type(data) == dict:
    #    output = {}
    #    for k in data.keys():
    #        output[k] = window_mean_2D_latlon(Lat, Lon, data[k], l, method)
    #    return output

    assert data.ndim >= 2, "The input data must has at leas 2 dimensions"
    assert (lat.shape == lon.shape), "lon and lat must have the same shape."
    assert (data.shape[-2:] == lat.shape), \
            "The last 2 dimensions of data mush be equal to lat & lon shape"
    assert type(data) in [np.ndarray, ma.MaskedArray], \
            "data must be an array or masked_array"

    wfunc = window_func(method)

    data_smooth = ma.masked_all(data.shape)

    #if interp == True:
    #    I, J = data.shape
    #    I, J = np.meshgrid(range(I), range(J))
    #    I = I.reshape(-1); J = J.reshape(-1)
    #else:
    #    I, J = np.nonzero(np.isfinite(data))

    #for i, j in zip(I, J):
    I, J = data.shape[-2:]
    for i in xrange(I):
        for j in xrange(J):
            data_smooth[..., i, j] = _convolve_2D_latlon(
                    lat[i,j], lon[i,j], lat, lon, l, wfunc, data)

    return data_smooth       


def _convolve_2D_latlon(lat0, lon0, lat, lon, l, winfunc, data):
    r = haversine(lat, lon, lat0, lon0)
    if len(r) > 0:
        # Index only the valid data that is inside the window
        #ind = (np.absolute(r) < l) & (~ma.getmaskarray(data))
        #w = winfunc(r[ind], l)
        w = winfunc(r, l)
        return _apply_convolve_2D(data, w)


def _apply_convolve_2D(data, w):
    """ Apply weights w into data

        This functions is usefull for arrays with more than 2D, so that the
          r and w are estimated the minimum ammount of times. It assumes that
          the weights (w) are applicable at the last 2 dimensions, and repeat
          the procedure to any number of previous dimensions. I.e., a 2D
          array is straight forward, while a 3D array, the procedure is
          repeated along the first dimension as n 2D arrays.
    """
    if data.ndim > 2:
        output = ma.masked_all(data.shape[:-2])
        I = data.shape[0]
        for i in xrange(I):
            output[i] = _apply_convolve_2D(data[i], w)
        return output

    #tmp = data[ind]*w
    tmp = data*w
    # Sum the weights only at the valid data positions.
    #wsum = w[ma.getmaskarray(tmp)].sum()
    wsum = w.sum()
    if wsum != 0:
        return (tmp).sum()/wsum


def wmean_2D_latlon(Lat, Lon, data, l, method='hamming', interp='False'):
    return wmean_2D_latlon_serial(Lat, Lon, data, l, method, interp)




"""
"""
# Gui, 27-06-2012
# Just an idea. Create a class of filtered data. A fake Masked Array object
#   which gives the output on demand, filtered.

try:
    import multiprocessing as mp
except:
    print "I couldn't import multiprocessing"

import numpy as N
import numpy
import numpy as np
from numpy import ma

from window_func import window_func
from distance import haversine

# ==== Bellow here, I need to do some serious work on ====

def window_mean(y,x=None,x_out=None,method="rectangular",boxsize=None):
    """Windowed means along 1-D array

    Input:
        - x [0,1,2,3,...] =>
        - x_out [x] =>
        - method [rectangular]:
            + rectangular => All data in window have same weight
        - boxsize [mean space] =>
    Output:

    Apply windowed means on a 1-D data array. Selecting adequate x_out
    and boxsize could define boxmeans or smooth filters. Method defines
    the weight method.

    An important point of this function is the ability to work with 
    unhomogenious spaced samples. Data ([1,2,3]) colected at [1,2,4] 
    times would be different if was collected at [1,2,3].
    """
    if(x==None):
        x=N.arange(N.size(y))

    if(x_out==None):
        x_out=x

    #y_out = N.zeros(N.size(x_out),N.float)
    y_out = ma.masked_all(x_out.shape)

    if(boxsize==None):
        # !!! Improve it! A better way than *1. ?!
        boxsize =(max(x)-min(x))/(N.size(x_out)*1.)

    half_boxsize = boxsize/2.

    #for x_i in x_out:
    for i in range(N.size(x_out)):
        x_i = x_out[i]


        # Higher window limit
        hi_limit = x_i+half_boxsize
        # Lower window limit
        lo_limit = x_i-half_boxsize
        # index of values inside window
        index = N.less_equal(x,hi_limit)*N.greater_equal(x,lo_limit)

        # !!! INSERT some type of check for minimum number of samples to be considered

        # x values on window around x_i
        x_tmp = N.compress(index,x)-x_i
        # y values on window
        y_tmp = N.compress(index,y)

        # weights in window according to x position
        weight = window_weight(x_tmp,boxsize,method)

        y_out[i] = N.sum(y_tmp*weight)

    return y_out

# To improve, the right to way to implement these filters are to define the halfpower cutoff, instead of an l dimension. Then the l dimension is defined on the function according to the weightning system for the adequate l.

def window_1Dmean_grid(data, l, method='hann', axis=0, parallel=False):
    """ A moving window mean filter applied to a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        The other types of filter consider the scales of each
          dimension. On this case it's considered a regular
          grid, so the filter can be based on the number of
          elements, and so be much optimized.

        l is in number of cells around the point being evaluated.
    """
    assert axis <= data.ndim, "Invalid axis!"

    if axis != 0:
        data_smooth = window_1Dmean_grid(data.swapaxes(0, axis), 
                l = l, 
                method = method, 
                axis = 0, 
                parallel = parallel)

        return data_smooth.swapaxes(0, axis)

    winfunc = window_func(method)
    r = np.arange(-np.floor(l/2),np.floor(l/2)+1)
    w = winfunc(r, l)

    data_smooth = ma.masked_all(data.shape)

    I = data.shape[0]
    norm = np.convolve(np.ones(I), w ,mode='same')
    if len(data.shape)==1:
	norm=numpy.convolve(numpy.ones(I),w,mode='same')
	data_smooth[:] = numpy.convolve(data[:],w,mode='same')/norm

    elif len(data.shape) == 2:
        I, J = data.shape
        for j in range(J):
            data_smooth[:,j] = np.convolve(data[:,j], w, mode='same')/norm

    elif len(data.shape) >2:
        I, J = data.shape[:2]
        for j in range(J):
                data_smooth[:,j] = window_1Dmean_grid(data[:,j], 
                        l = l, 
                        method = method, 
                        axis=0, 
                        parallel = parallel)

    try:
        data_smooth.mask = data.mask
    except:
        pass

    return data_smooth


# ----
def get_halfpower_period(data, filtered, dt):
    """ Returns the gain per frequency
    """
    nt, ni, nj = data.shape
    gain = ma.masked_all((nt, ni, nj))
    for i in range(ni):
        for j in range(nj):
	    if ~filtered[:,i,j].mask.all():
	        gain[:,i,j] = np.absolute(np.fft.fft(filtered[:,i,j]-filtered[:,i,j].mean())) / np.absolute(np.fft.fft(data[:,i,j]-data[:,i,j].mean()))
    gain_median = ma.masked_all(nt)
    gain_25 = ma.masked_all(nt)
    gain_75 = ma.masked_all(nt)
    # Run for each frequency, which are in the same number of timesteps
    from scipy.stats import scoreatpercentile
    for t in range(nt):
        #gain_median[t] = numpy.median(gain[t,:,:].compressed()[numpy.isfinite(gain[t,:,:].compressed())])
        #tmp = gain[t,:,:].compressed()[numpy.isfinite(gain[t,:,:].compressed())]
        #gain_median[t] = scoreatpercentile(tmp,50)
        gain_median[t] = ma.median(gain[t])
        #gain_25[t] = scoreatpercentile(tmp,25)
        #gain_75[t] = scoreatpercentile(tmp,75)

    freq = np.fft.fftfreq(nt)/dt.days

    halfpower_period = 1./freq[np.absolute(gain_median-0.5).argmin()]


    #from scipy.interpolate import UnivariateSpline
    #s = UnivariateSpline(gain_median[numpy.ceil(nt/2.):], -freq[numpy.ceil(nt/2.):], s=1)
    #xs = -freq[numpy.ceil(nt/2.):]
    #ys = s(xs)

    #import rpy2.robjects as robjects
    #smooth = robjects.r['smooth.spline'](robjects.FloatVector(gain_median[numpy.ceil(nt/2.):]),robjects.FloatVector(-freq[numpy.ceil(nt/2.):]),spar=.4)
    ##smooth = robjects.r['smooth.spline'](robjects.FloatVector(-freq[numpy.ceil(nt/2.):]),robjects.FloatVector(gain_median[numpy.ceil(nt/2.):]),spar=.4)
    #s_interp = robjects.r['predict'](smooth,x=0.5)
    ##halfpower_period = 1./s_interp.rx2['y'][0]
    #halfpower_period = 1./s_interp.rx2(2)[0]

    #smooth = robjects.r['smooth.spline'](robjects.FloatVector(-freq[numpy.ceil(nt/2.):]),robjects.FloatVector(gain_median[numpy.ceil(nt/2.):]),spar=.4)
    #s_interp = robjects.r['predict'](smooth, x = robjects.FloatVector(-freq[numpy.ceil(nt/2.):]))

    #print "Filter half window size: %s" % l
    #print "Half Power Period: %s" % halfpower_period
    #self.halfpower_period = halfpower_period

    return halfpower_period



#pylab.plot(-freq[numpy.ceil(nt/2.):],  gain_median[numpy.ceil(nt/2.):]  )
#pylab.plot(1./halfpower_period, 0.5,'rx')
#pylab.show()

# In the future implement a check to show the average effect in the
#   spectrum. I.e. estimate fft before and after and compare the difference
#I,J,K = ssh.shape
##fft_input = ma.masked_all((I,J,K))
##fft_output = ma.masked_all((I,J,K))
#tmp = ma.masked_all((I,J,K))
#tmp2 = ma.masked_all(I)
#for j in range(J):
#    for k in range(K):
#        if ssh[:,j,k].mask.any()==False:
#            #fft_input[:,j,k] = numpy.fft.fft(ssh[:,j,k])
#            fft_input = numpy.absolute(numpy.fft.fft(ssh[:,j,k]))
#            #fft_output[:,j,k] = numpy.fft.fft(output[:,j,k])
#            fft_output = numpy.absolute(numpy.fft.fft(output[:,j,k]))
#            tmp[:,j,k] = (fft_input-fft_output)/fft_input
#for i in range(I):
#    tmp2[i] = tmp[i].mean()




# -*- coding: utf-8 -*-
# cython: profile=False
# filename: window_func_scalar.pyx
# http://docs.cython.org/src/userguide/numpy_tutorial.html#numpy-tutorial

import numpy as np

cimport numpy as np
from libc.math cimport cos, abs, M_PI


cdef double twopi = 2*M_PI


def window_func_scalar(method='hamming'):
    """ Select the weight function
    """
    if method == 'hamming':
        return _weight_hamming_scalar

    elif method == 'hann':
        return _weight_hann_scalar

    elif method == 'blackman':
        return _weight_blackman_scalar

    elif method == 'boxcar':
        return _weight_boxcar_scalar

    else:
        raise Exception("%s is not available" % method)

    #elif method == 'triangular':
    #    winfunc = _weight_triangular_scalar


# Hamming
def _weight_hamming_scalar(double r, double l):

    if abs(r) > l/2.:
        return 0

    return 0.54 + 0.46*cos(twopi*r/l)


# Hann
def _weight_hann_scalar(double r, double l):
    """ Need to double check and create a test
    """
    if abs(r) > l/2.:
        return 0

    return 0.5*(1 + cos(twopi*r/l))


# blackman
def _weight_blackman_scalar(double r, double l):
    """ Need to double check and create a test
    """
    if abs(r) > l/2.:
        return 0

    cdef double theta = twopi*r/l
    # fase lag-> sign change
    return 0.42 +  0.5*cos(theta) + 0.08*cos(2*theta)


# triangular
#def _weight_triangular_scalar(double r, double l):
#    """ Need to double check and create a test
#    """
#    if abs(r) > l/2.:
#        return 0
#
#    return 1 - abs(2*r/l)


# boxcar or rectangular
def _weight_boxcar_scalar(double r, double l):
    """ Need to double check and create a test
    """
    if abs(r) > l/2.:
        return 0

    return 1



# -*- coding: utf-8 -*-
# cython: profile=False

"""

    Wasn't this below supposed to be faster than from i in xrange()?
    #for i from 0 < = i < I:
    #    for j from 0 < = j < J:
"""

import numpy as np

cimport numpy as np
from libc.math cimport cos, sin, asin, sqrt, M_PI


np.import_array()

DTYPE = np.float
ctypedef np.float_t DTYPE_t

cdef double DEG2RAD = M_PI/180.
cdef float AVG_EARTH_RADIUS = 6371000  # in m
cdef float TWO_AVG_EARTH_RADIUS = 2 * AVG_EARTH_RADIUS # in m



def find_closer_then(lat, lon, lat_c, lon_c, llimit, method="haversine"):
    """
    """
    assert lat.shape == lon.shape, "lat & lon must have same shape"
    #assert (len(lat_c) == 1) & (len(lon_c) == 1)

    deg_limit = np.rad2deg(llimit/AVG_EARTH_RADIUS)
    possible = np.nonzero(abs(lat - lat_c) <= deg_limit)
    L = haversine(lat[possible], lon[possible], lat_c, lon_c)
    ind_closer = L<llimit
    L = L[ind_closer]
    ind = []
    # Repeat for each dimension.
    for p in possible:
        ind.append(p[ind_closer])

    return ind, L


cdef double _haversine_scalar(double lat, double lon, double lat_c,
        double lon_c):
    """ Haversine, gives the grat circle distance in a sphere
    """

    cdef double d
    cdef double h

    lat = lat * DEG2RAD
    lon = lon * DEG2RAD
    lat_c = lat_c * DEG2RAD
    lon_c = lon_c * DEG2RAD

    #sinlat = sin( (lat - lat_c) / 2.)
    #sinlon = sin( (lon - lon_c) / 2.)

    d = sin( (lat - lat_c) / 2.) ** 2 + \
            cos(lat) * cos(lat_c) * sin( (lon - lon_c) / 2) ** 2
    h = TWO_AVG_EARTH_RADIUS * asin(sqrt(d))

    return h


def haversine_scalar(lat, lon, lat_c, lon_c):
    """ Python wrapper for _haversine_scalar()
    """
    return _haversine_scalar(lat, lon, lat_c, lon_c)


def haversine_1Darray(np.ndarray[DTYPE_t, ndim=1] lat,
        np.ndarray[DTYPE_t, ndim=1] lon, double lat_c, double lon_c):
    """ Harversine in a 1D numpy.array

        Inputs:
            lat [1D np.array]: Latitudes
            lon [1D np.array]: Longitudes
            lat_c [float]: Latitude of reference
            lon_c [float]: Longitude of reference
    """

    # Why does this assert fails?
    #assert lat.shape == lon.shape

    cdef unsigned int i
    cdef unsigned int I = lat.shape[0]
    cdef np.ndarray[DTYPE_t, ndim=1] h = np.empty(I)
    cdef double dlat, dlon, d
    cdef double cos_lat_c

    lat_c = lat_c * DEG2RAD
    lon_c = lon_c * DEG2RAD

    cos_lat_c = cos(lat_c)

    for i in xrange(I):
        lat[i] = lat[i] * DEG2RAD
        lon[i] = lon[i] * DEG2RAD

        dlat = lat[i] - lat_c
        dlon = lon[i] - lon_c

        d = sin(dlat / 2) ** 2 + \
            cos(lat[i]) * cos_lat_c * sin(dlon / 2) ** 2
        h[i] = TWO_AVG_EARTH_RADIUS * asin(sqrt(d))

    return h


def haversine_2Darray(np.ndarray[DTYPE_t, ndim=2] lat,
        np.ndarray[DTYPE_t, ndim=2] lon, double lat_c, double lon_c):
    """ Harversine in a 2D numpy.array

        Inputs:
            lat [2D np.array]: Latitudes
            lon [2D np.array]: Longitudes
            lat_c [float]: Latitude of reference
            lon_c [float]: Longitude of reference
    """

    cdef unsigned int i, j
    cdef unsigned int I = lat.shape[0]
    cdef unsigned int J = lat.shape[1]
    cdef np.ndarray[DTYPE_t, ndim=2] h = np.empty((I, J))
    cdef double dlat, dlon, d
    cdef double cos_lat_c

    lat_c = lat_c * DEG2RAD
    lon_c = lon_c * DEG2RAD

    cos_lat_c = cos(lat_c)

    for i in xrange(I):
        for j in xrange(J):
            lat[i, j] = lat[i, j] * DEG2RAD
            lon[i, j] = lon[i, j] * DEG2RAD

            dlat = lat[i, j] - lat_c
            dlon = lon[i, j] - lon_c

            d = sin(dlat / 2) ** 2 + \
                    cos(lat[i, j]) * cos_lat_c * sin(dlon / 2) ** 2
            h[i, j] = TWO_AVG_EARTH_RADIUS * asin(sqrt(d))

    return h


def haversine(lat, lon, lat_c, lon_c):

    if (type(lat) == np.ndarray) & (type(lon) == np.ndarray):
        if (lat.ndim == 1) & (lon.ndim == 1):
            return haversine_1Darray(lat, lon, lat_c, lon_c)

        if (lat.ndim == 2) & (lon.ndim == 2):
            return haversine_2Darray(lat, lon, lat_c, lon_c)



# -*- coding: utf-8 -*-

import numpy as np
from numpy import ma
from window_func import window_func


def wmean_2D_serial(x, y, data, l, method='hamming', interp=False):
    """
        - interp
        - split in two solutions, array and masked array
        - Allow nD input arrays
    """
    assert data.ndim >= 2, "The input data must has at leas 2 dimensions"
    assert (x.shape == y.shape), "x and y must have the same shape."
    assert (data.shape[-2:] == x.shape), \
            "The last 2 dimensions of data mush be equal to x & y shape"
    assert type(data) in [np.ndarray, ma.MaskedArray], \
            "data must be an array or masked_array"

    if type(data) is np.ndarray:
        data_smooth = np.empty(data.shape)
    else:
        data_smooth = ma.masked_all(data.shape)

    # ----
    if data.ndim > 2:
        for i in xrange(data.shape[0]):
            data_smooth[i] = wmean_2D_serial(x, y, data[i], l, method, interp)
        return data_smooth
    # Below here it is expected only 2D arrays
    # ----
    winfunc = window_func(method)

    #I, J = data.shape[-2:]
    #for i in xrange(I):
    #    for j in xrange(J):
    if interp == True:
        I, J = data.shape
        I, J = np.meshgrid(range(I), range(J))
        I = I.reshape(-1); J = J.reshape(-1)
    else:
        I, J = np.nonzero(~ma.getmaskarray(data))

    for i, j in zip(I, J):
        data_smooth[..., i, j] = _convolve_2D(x[i,j], y[i,j], x, y, l,
                winfunc, data)

    ind_nan = np.isnan(data_smooth)
    if ind_nan.any():
        data_smooth.mask[ind_nan] = True

    return data_smooth


def _convolve_2D(x0, y0, x, y, l, winfunc, data):
    r = ( (x-x0)**2 + (y-y0)**2 )**0.5
    if len(r) > 0:
        # Index only the valid data that is inside the window
        #ind = (np.absolute(r) < l) & (~ma.getmaskarray(data))
        #w = winfunc(r[ind], l)
        w = winfunc(r, l)
        return _apply_convolve_2D(data, w)


def _apply_convolve_2D(data, w):
    """ Apply weights w into data

        This functions is usefull for arrays with more than 2D, so that the
          r and w are estimated the minimum ammount of times. It assumes that
          the weights (w) are applicable at the last 2 dimensions, and repeat
          the procedure to any number of previous dimensions. I.e., a 2D
          array is straight forward, while a 3D array, the procedure is
          repeated along the first dimension as n 2D arrays.
    """
    if data.ndim > 2:
        output = ma.masked_all(data.shape[:-2])
        I = data.shape[0]
        for i in xrange(I):
            output[i] = _apply_convolve_2D(data[i], w)
        return output

    ind = (w != 0) & (~ma.getmaskarray(data))
    tmp = data[ind]*w[ind]
    wsum = w[ind].sum()
    if wsum != 0:
        return (tmp).sum()/wsum


def wmean_2D(x, y, data, l, method='hamming', interp=False):

    print("I'm not ready yet to run in parallel. I'll do serial instead.")
    return wmean_2D_serial(x, y, data, l, method, interp)




"""
"""
# Gui, 27-06-2012
# Just an idea. Create a class of filtered data. A fake Masked Array object
#   which gives the output on demand, filtered.

try:
    import multiprocessing as mp
except:
    print "I couldn't import multiprocessing"

import numpy as np
from numpy import ma

from window_func import window_func


def wmean_1D_serial(data, l, t=None, method='hann', axis=0, interp=False):
    """ A moving window mean filter, not necessarily a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        It's not optimized for a regular grid.

        Input:
            - data: np.array or ma.maked_array, nD
            - l: is the size of the filter, in the same unit
                of the t.
	    - t: is the scale of the choosed axis, 1D. If not
                defined, it will be considered a sequence.
            - method: ['hann', 'hamming', 'blackman']
                Defines the weight function type
            - axis: Dimension which the filter will be applied
    """
    #assert type(data)) in [np.ndarray, ma.MaskedArray]
    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        print("Will temporary swapaxes")
        data_smooth = wmean_1D_serial(data.swapaxes(0, axis),
            l = l,
            t = t,
            method = method,
            axis = 0,
            interp = interp)

        return data_smooth.swapaxes(0, axis)
    # Below here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
	t = np.arange(data.shape[axis])

    assert t.shape == (data.shape[axis],), "Invalid size of t."

    if type(data) is np.ndarray:
        data_smooth = np.empty(data.shape)
    else:
        data_smooth = ma.masked_all(data.shape)

    # ----
    if data.ndim > 1:
        for i in range(data.shape[1]):
            data_smooth[:,i] = wmean_1D_serial(data[:,i],
                l = l,
                t = t,
                method = method,
                axis = 0,
                interp = interp)

        return data_smooth

    # ----
    # Here on it is expected data.ndim == 1

    wfunc = window_func(method)

    if interp is True:
        I = range(len(t))
    else:
        (I,) = np.nonzero(~ma.getmaskarray(data))

    for i in I:
        dt = t - t[i]
        w = wfunc(dt, l)
        ind = (w != 0) & (~ma.getmaskarray(data))
        if ind.any():
            tmp = data[ind]*w[ind]
            wsum = w[ind].sum()
            if wsum != 0:
                data_smooth[i] = (tmp).sum()/wsum

    return data_smooth


def wmean_1D(data, l, t=None, method='hann', axis=0, interp = False):
    """ A moving window mean filter, not necessarily a regular grid.

        It is equivalent to wmean_1D_serial but run in parallel with
          multiprocesing for higher efficiency.

        Check wmean_1D_serial documentation for the inputs and other
          details.
    """
    #assert type(data)) in [np.ndarray, ma.MaskedArray]
    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        data_smooth = wmean_1D(data.swapaxes(0, axis),
            l = l,
            t = t,
            method = method,
            axis = 0,
            interp = interp)

        return data_smooth.swapaxes(0, axis)
    # Below here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
	t = np.arange(data.shape[axis])

    assert t.shape == (data.shape[axis],), "Invalid size of t."

    # ----
    # Only one dimensions usually means overhead to run in parallel.
    if data.ndim==1:
        data_smooth = wmean_1D_serial(data, l, t=t, method=method, axis=axis,
                interp=interp)
        return data_smooth
    # ----

    npes = 2 * mp.cpu_count()
    pool = mp.Pool(npes)
    results = []
    I = data.shape[1]
    for i in range(I):
        results.append(pool.apply_async(wmean_1D_serial, \
                (data[:,i], l, t, method, 0, interp)))
    pool.close()

    # Collecting the results.
    if type(data) is np.ndarray:
        data_smooth = np.empty(data.shape)
    else:
        data_smooth = ma.masked_all(data.shape)

    for i, r in enumerate(results):
        data_smooth[:,i] = r.get()
    pool.terminate()

    return data_smooth


def _convolve_1D(t0, t, l, winfunc, data):
    """ Effectively apply 1D moving mean along 1D array

        This is not exactly a convolution.

        Support function to wmean_1D

        ATENTION, in the future I should use l/2. in the index for most of
          the weighting windows types.
    """
    dt = t - t0
    # Index only the valid data that is inside the window
    #ind = (np.absolute(dt) < l) & (~ma.getmaskarray(data))
    #ind = np.nonzero( ind )
    #w = winfunc(dt[ind], l)
    w = winfunc(dt, l)
    #return (data[ind] * w).sum() / (w.sum())
    #ind = w != 0
    #return _apply_convolve_1D(data[ind], w[ind])

    return _apply_convolve_1D(data, w)


def _apply_convolve_1D(data, w):
    if data.ndim > 1:
        output = ma.masked_all(data.shape[1:])
        for i in xrange(data.shape[1]):
            output[i] = _apply_convolve_1D(data[:,i], w)
        return output
    ind = (~ma.getmaskarray(data))
    tmp = data[ind]*w
    wsum = w[ind].sum()
    if wsum != 0:
        return (tmp).sum()/wsum


def wmean_bandpass_1D_serial(data, lshorterpass, llongerpass, t=None,
        method='hann', axis=0):
    """ Equivalent to wmean_1D_serial, but it is a bandpass

        Input:
            - data: np.array or ma.maked_array, nD
            - lshorterpass: The size of the highpass filter, i.e. shorter
                wavelenghts are preserved. It is in the same unit of t.
            - llongerpass: The size of the lowpass filter, i.e.longer
                wavelenghts are preserved. It is in the same unit of t.
	    - t: is the scale of the choosed axis, 1D. If not
                defined, it will be considered a sequence.
            - method: ['hann', 'hamming', 'blackman']
                Defines the weight function type
            - axis: Dimension which the filter will be applied
    """

    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        data_smooth = wmean_bandpass_1D_serial(data.swapaxes(0, axis),
                lshorterpass = lshorterpass,
                llongerpass = llongerpass,
                t = t,
                method = method,
                axis = 0)

        return data_smooth.swapaxes(0, axis)
    # Below here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
	t = np.arange(data.shape[axis])

    assert t.shape == (data.shape[axis],), "Invalid size of t."

    # ----
    winfunc = window_func(method)

    data_smooth = ma.masked_all(data.shape)

    if data.ndim==1:
        (I,) = np.nonzero(~ma.getmaskarray(data))
        for i in I:
            # First remove the high frequency
            tmp = _convolve_1D(t[i], t, llongerpass, winfunc, data)
            # Then remove the low frequency
            data_smooth[i] = tmp - \
                    _convolve_1D(t[i], t, lshorterpass, winfunc, tmp)

    else:
        I = data.shape[1]
        for i in range(I):
            data_smooth[:,i] = wmean_bandpass_1D_serial(datai[:,i],
                    lshorterpass, llongerpass, t, method, axis)

    return data_smooth


def wmean_bandpass_1D(data, lshorterpass, llongerpass, t=None,
        method='hann', axis=0):
    """ A bandpass moving window filter, for not necessarily regular grids.

        Equivalent to wmean_bandpass_1D_serial, but it runs in parallel
          with multiprocessing for higher efficiency.

        Check wmean_bandpass_1D_serial documentation for the inputs and
            other details.
    """

    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        data_smooth = wmean_bandpass_1D(data.swapaxes(0, axis),
                lshorterpass = lshorterpass,
                llongerpass = llongerpass,
                t = t,
                method = method,
                axis = 0)

        return data_smooth.swapaxes(0, axis)
    # Below here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
	t = np.arange(data.shape[axis])

    assert t.shape == (data.shape[axis],), "Invalid size of t."

    # ----
    # Only one dimensions usually means overhead to run in parallel.
    if data.ndim==1:
        data_smooth = wmean_bandpass_1D_serial(data, lshorterpass,
                llongerpass, t, method, axis)
        return data_smooth
    # ----

    npes = 2 * mp.cpu_count()
    pool = mp.Pool(npes)
    results = []
    I = data.shape[1]
    for i in range(I):
        results.append(pool.apply_async(wmean_bandpass_1D_serial, \
                (data[:,i], lshorterpass, llongerpass, t, method, axis)))
    pool.close()

    # Collecting the results.
    data_smooth = ma.masked_all(data.shape)
    for i, r in enumerate(results):
        data_smooth[:,i] = r.get()
    pool.terminate()

    return data_smooth
